﻿using System.Data;
using System.Data.SqlClient;
using System.Web.Configuration;

using TestProject.Models;

namespace TestProject
{
    /// <summary>
    /// 사용자 저장소
    /// </summary>
    public class UserRepository
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// SQL 연결
        /// </summary>
        private SqlConnection connection;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - UserRepository()

        /// <summary>
        /// 생성자
        /// </summary>
        public UserRepository()
        {
            this.connection = new SqlConnection();

            this.connection.ConnectionString = WebConfigurationManager.ConnectionStrings["ConnectionString"].ConnectionString;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 사용자 추가하기 - AddUser(userID, password)

        /// <summary>
        /// 사용자 추가하기
        /// </summary>
        /// <param name="userID">사용자 ID</param>
        /// <param name="password">패스워드</param>
        public void AddUser(string userID, string password)
        {
            SqlCommand command = new SqlCommand();

            command.Connection  = this.connection;
            command.CommandText = "WriteUser";
            command.CommandType = CommandType.StoredProcedure;

            command.Parameters.AddWithValue("@UserID"  , userID  );
            command.Parameters.AddWithValue("@Password", password);

            this.connection.Open();

            command.ExecuteNonQuery();

            this.connection.Close();
        }

        #endregion
        #region 사용자 구하기 - GetUser(userID)

        /// <summary>
        /// 사용자 구하기
        /// </summary>
        /// <param name="userID">사용자 ID</param>
        /// <returns>사용자</returns>
        public UserModel GetUser(string userID)
        {
            UserModel user = new UserModel();

            SqlCommand command = new SqlCommand();

            command.Connection  = this.connection;
            command.CommandText = "SELECT * FROM dbo.[User] WHERE UserID = @UserID";
            command.CommandType = CommandType.Text;

            command.Parameters.AddWithValue("@UserID", userID);

            this.connection.Open();

            IDataReader reader = command.ExecuteReader();

            if(reader.Read())
            {
                user.ID       = reader.GetInt32(0);
                user.UserID   = reader.GetString(1);
                user.Password = reader.GetString(2);
            }

            this.connection.Close();

            return user; 
        }

        #endregion
        #region 사용자 수정하기 - UpdateUser(id, userID, password)

        /// <summary>
        /// 사용자 수정하기
        /// </summary>
        /// <param name="id">ID</param>
        /// <param name="userID">사용자 ID</param>
        /// <param name="password">패스워드</param>
        public void UpdateUser(int id, string userID, string password)
        {
            SqlCommand command = new SqlCommand();

            command.Connection  = this.connection;
            command.CommandText = "UpdateUser";
            command.CommandType = CommandType.StoredProcedure;

            command.Parameters.AddWithValue("@UserID"  , userID  );
            command.Parameters.AddWithValue("@Password", password);
            command.Parameters.AddWithValue("@ID"      , id     );

            this.connection.Open();

            command.ExecuteNonQuery();

            this.connection.Close();
        }

        #endregion
        #region 검증된 사용자 여부 구하기 - IsValidUser(userID, password)

        /// <summary>
        /// 검증된 사용자 여부 구하기
        /// </summary>
        /// <param name="userID">사용자 ID</param>
        /// <param name="password">패스워드</param>
        /// <returns>검증된 사용자 여부</returns>
        public bool IsValidUser(string userID, string password)
        {
            bool result = false;

            this.connection.Open();

            SqlCommand command = new SqlCommand();

            command.Connection  = this.connection;
            command.CommandText = "SELECT * FROM dbo.[User] WHERE UserID = @UserID AND Password = @Password";
            command.CommandType = CommandType.Text;

            command.Parameters.AddWithValue("@UserID"  , userID  );
            command.Parameters.AddWithValue("@Password", password);

            SqlDataReader reader = command.ExecuteReader();

            if(reader.Read())
            {
                result = true;
            }

            reader.Close();

            this.connection.Close();

            return result;
        }

        #endregion
    }
}